import 'core.dart';

class ExceptionCode {
  final String _exceptionCode;

  const ExceptionCode._(this._exceptionCode);

  static const kNoAuthUserExceptionCode = ExceptionCode._('NO_AUTH_USER');
  static const kNoCacheDataExceptionCode = ExceptionCode._('NO_CACHE_DATA');
  static const kFailedTransactionExceptionCode =
      ExceptionCode._('FAILED_TRANSACTION');
  static const kUndefinedExceptionCode = ExceptionCode._('UNDEFINED');
  static const kConnectionTimeoutExceptionCode =
      ExceptionCode._('CONNECTION_TIMEOUT');
  static const kNoInternetConnectionExceptionCode =
      ExceptionCode._('NO_INTERNET_CONNECTION');

  static ExceptionCode parse(String type) {
    return ExceptionCode._(type);
  }

  @override
  String toString() => _exceptionCode;

  bool operator ==(o) =>
      (o is ExceptionCode) && o._exceptionCode == this._exceptionCode;

  @override
  int get hashCode => _exceptionCode.hashCode;
}

class ExceptionMessages {
  static final kNoAuthUser = AuthException(
    code: ExceptionCode.kNoAuthUserExceptionCode,
    message: 'No authenticated m4e user account',
  );

  static final kNoCacheDataException = CacheException(
    code: ExceptionCode.kNoCacheDataExceptionCode,
    message: 'No cache data!',
  );

  static final kFailedTransactionException = ServerException(
    code: ExceptionCode.kFailedTransactionExceptionCode,
    message: 'Transaction failed to complete!',
  );

  static final kUndefinedException = ServerException(
    code: ExceptionCode.kUndefinedExceptionCode,
    message: 'Something went wrong!',
  );

  static final kConnectionTimeOutException = ServerException(
    code: ExceptionCode.kConnectionTimeoutExceptionCode,
    message: 'Connection timeout!',
  );

  static final kNoInternetConnectionException = ServerException(
    code: ExceptionCode.kNoInternetConnectionExceptionCode,
    message: 'No internet connection!',
  );
}

class ClientRequest {
  /// connection timeout
  static const kConnectionTimeout = 60000;
  static const kRecieveTimeout = 60000;
}

class Persistence {
  static const kM4eAuthToken = 'M4E_AUTH_TOKEN';
  static const kM4eUserWallets = 'M4E_USER_WALLETS';
  static const kM4eUserTransactionLogs = 'M4E_USER_TRANSACTION_LOGS';
}
