import 'dart:io';

import 'package:dio/dio.dart';
import 'package:flutter/foundation.dart';

import '../../data/api/auth_api.dart';

class M4eHeaderInterceptors extends InterceptorsWrapper {
  final Dio _dio;
  final AuthApi _authApi;

  M4eHeaderInterceptors({
    @required Dio dio,
    @required AuthApi authApi,
  })  : assert(dio != null),
        assert(authApi != null),
        _dio = dio,
        _authApi = authApi;

  @override
  Future onRequest(RequestOptions options) async {
    String _accessToken = (await _authApi.getAuthTokenFromCache())?.token;

    options.headers.addAll({
      'Content-type': 'application/json',
      HttpHeaders.authorizationHeader: 'Bearer $_accessToken'
    });

    return options;
  }

  @override
  Future onError(DioError dioError) async {
    int _responseCode = dioError.response.statusCode;
    final _existingAuthToken = await _authApi.getAuthTokenFromCache();

    if (_existingAuthToken?.token != null && _responseCode == 401) {
      _dio.interceptors.requestLock.lock();
      _dio.interceptors.responseLock.lock();
      print('Requesting for new token');

      try {
        final _authToken =
            await _authApi.refreshAccessToken(_existingAuthToken);

        await _authApi.cacheAuthToken(_authToken);

        RequestOptions options = dioError.response.request;
        _dio.interceptors.requestLock.unlock();
        _dio.interceptors.responseLock.unlock();
        return _dio.request(options.path, options: options);
      } catch (e) {
        print('Interceptor error ===> $e');
        super.onError(e);
      }
    } else {
      super.onError(dioError);
    }
  }
}
