import 'package:equatable/equatable.dart';
import 'package:flutter/foundation.dart';

class ConfigFlavour {
  final String _type;

  const ConfigFlavour._(this._type);

  static const production = ConfigFlavour._('PRODUCTION');
  static const staging = ConfigFlavour._('STAGING');
  static const development = ConfigFlavour._('DEVELOPMENT');

  static ConfigFlavour parse(String type) {
    return ConfigFlavour._(type);
  }

  @override
  operator ==(cmp) => (cmp is ConfigFlavour) && cmp._type == _type;

  @override
  int get hashCode => _type.hashCode;

  @override
  String toString() => _type;
}

class SDKConfig extends Equatable {
  final String appId;
  final ConfigFlavour flavour;
  final String baseUrl;

  const SDKConfig._(
    this.appId,
    this.flavour,
    this.baseUrl,
  );

  // TODO: implement other sdk-config envs.
  // TODO: update base-urls to m4e server
  static const _developmentBaseUrl = 'https://vsla-dev.herokuapp.com';

  factory SDKConfig.development({@required String appId}) {
    assert(appId != null, 'SDKConfig [appId] should not be null');
    return SDKConfig._(appId, ConfigFlavour.development, _developmentBaseUrl);
  }

  @override
  String toString() =>
      'SDKConfig(appId: $appId, flavour: $flavour, baseUrl: $baseUrl)';

  @override
  List<Object> get props => [appId, flavour, baseUrl];
}
