import 'package:equatable/equatable.dart';
import 'package:flutter/foundation.dart';

import '../../core/core.dart';
import '../../models/models.dart';

enum TransactionStatus { pending, failed, success }

class TransactionMeta extends Equatable {
  final String callbackUrl;
  final String purpose;
  final String data;

  const TransactionMeta._(this.callbackUrl, this.purpose, this.data);

  factory TransactionMeta({
    @required String callbackUrl,
    @required String purpose,
    @required String data,
  }) {
    assert(callbackUrl != null,
        'TransactionMeta [callbackUrl] should not be null');
    assert(purpose != null, 'TransactionMeta [purpose] should not be null');
    assert(data != null, 'TransactionMeta [data] should not be null');
    return TransactionMeta._(callbackUrl, purpose, data);
  }

  Map<String, dynamic> toMap() {
    return {
      'callbackUrl': this.callbackUrl,
      'purpose': this.purpose,
      'data': this.data,
    };
  }

  @override
  List<Object> get props => [
        callbackUrl,
        purpose,
        data,
      ];

  @override
  String toString() =>
      'TransactionMeta(callbackUrl: $callbackUrl, purpose: $purpose, data: $data)';
}

/// m4e transaction object for making transactions with m4e
///
/// Complete all fields for a valid transaction
///
class M4eTransactionForm extends Equatable {
  final Wallet from;
  final Wallet to;
  final Amount amount;
  final DateTime at;
  final TransactionMeta meta;
  final TransactionStatus status;

  const M4eTransactionForm._(
      this.from, this.to, this.amount, this.at, this.meta, this.status);

  factory M4eTransactionForm({
    @required Wallet from,
    @required Wallet to,
    @required Amount amount,
    @required DateTime at,
    @required TransactionMeta meta,
    TransactionStatus status = TransactionStatus.pending,
  }) {
    assert(from != null, 'Transaction [from] wallet should not be null');
    assert(to != null, 'Transaction [to] wallet should not be null');
    assert(from != to,
        'Transaction [from] and [to] wallets should not be the same');
    assert(amount != null, 'Transaction [amount] should not be null');
    assert(at != null, 'Transaction [at] should not be null');
    assert(meta != null, 'Transaction [meta] should not be null');
    return M4eTransactionForm._(from, to, amount, at, meta, status);
  }

  Map<String, dynamic> toMap() {
    final flow = {
      'from': this.from.toMap(),
      'to': this.to.toMap(),
      'amount': this.amount.toMap(),
      'at': DateTime.now().toTimestampSinceEpoch()
      // TODO: add meta to object
    };

    return {'flow': flow, 'txType': 'CASH_IN'};
  }

  @override
  List<Object> get props => [
        from,
        to,
        amount,
        at,
        meta,
        status,
      ];

  @override
  String toString() {
    return 'M4eTransactionForm(from: $from, to: $to, amount: $amount, at: $at, meta: $meta, status: $status)';
  }
}
