import 'package:equatable/equatable.dart';
import 'package:flutter/foundation.dart';

import '../core/core.dart';
import '../models/models.dart';

class TransactionLog extends Equatable {
  final Wallet from;
  final Wallet to;
  final Amount amount;
  final DateTime at;
  final TransactionType type;

  const TransactionLog({
    @required this.from,
    @required this.to,
    @required this.amount,
    @required this.at,
    @required this.type,
  });

  factory TransactionLog.fromJson(Map<String, dynamic> json) {
    return TransactionLog(
      from: Wallet.fromJson(json['flow']['from']),
      to: Wallet.fromJson(json['flow']['to']),
      amount: Amount.fromJson(json['flow']['amount']),
      at: DateTimeConverter.fromTimestampSinceEpoch(json['flow']['at']),
      type: TransactionType.parse(json['txType']),
    );
  }

  Map<String, dynamic> toMap() {
    final flow = {
      'from': this.from.toMap(),
      'to': this.to.toMap(),
      'amount': this.amount.toMap(),
      'at': this.at.toTimestampSinceEpoch(),
    };

    return {'flow': flow, 'txType': this.type};
  }

  @override
  List<Object> get props => [
        from,
        to,
        amount,
        at,
        type,
      ];

  @override
  String toString() {
    return 'TransactionLog(from: $from, to: $to, amount: $amount, at: $at, type: $type)';
  }
}

class TransactionLogList extends Equatable {
  final List<TransactionLog> transactionLogs;

  const TransactionLogList({@required this.transactionLogs});

  factory TransactionLogList.fromJson(Map<String, dynamic> json) {
    return TransactionLogList(
      transactionLogs:
          (json['data'] as List)?.map((e) => e['transaction'])?.toList() ?? [],
    );
  }

  Map<String, dynamic> toMap() {
    return {
      'data': this
              .transactionLogs
              ?.map((e) => {'transaction': e.toMap()})
              ?.toList() ??
          []
    };
  }

  @override
  List<Object> get props => [transactionLogs];

  @override
  String toString() => 'TransactionLogList(transactionLogs: $transactionLogs)';
}
