import 'package:equatable/equatable.dart';
import 'package:flutter/foundation.dart';

import '../core/core.dart';

class WalletType {
  final String _type;

  const WalletType._(this._type);

  // Template
  static const bascule = WalletType._('BASCULE');

  static WalletType parse(String type) {
    return WalletType._(type);
  }

  @override
  operator ==(cmp) => (cmp is WalletType) && cmp._type == _type;

  @override
  int get hashCode => _type.hashCode;

  @override
  String toString() => _type;
}

class WalletInfo extends Equatable {
  final String label;
  final String currency;
  final WalletType type;

  const WalletInfo._(this.label, this.currency, this.type);

  factory WalletInfo({
    @required String label,
    @required String currency,
    WalletType type = WalletType.bascule,
  }) {
    assert(label != null, 'WalletInfo [label] should not be null');
    assert(currency != null, 'WalletInfo [currency] should not be null');
    assert(type != null, 'WalletInfo [type] should not be null');
    return WalletInfo._(label, currency, type);
  }

  factory WalletInfo.fromJson(Map<String, dynamic> json) {
    return WalletInfo(
      label: json['label'],
      currency: json['currency'],
      type: WalletType.parse(json['type']),
    );
  }

  @override
  List<Object> get props => [
        label,
        currency,
        type,
      ];

  @override
  String toString() =>
      'WalletInfo(label: $label, currency: $currency, type: $type)';
}

class Wallet extends Equatable {
  final UniqueId id;
  final WalletInfo walletInfo;

  const Wallet._(this.id, this.walletInfo);

  factory Wallet({@required UniqueId id, @required WalletInfo walletInfo}) {
    assert(id != null, 'Wallet [id] should not be null');
    assert(walletInfo != null, 'Wallet [walletInfo] should not be null');
    return Wallet._(id, walletInfo);
  }

  factory Wallet.fromJson(Map<String, dynamic> json) {
    return Wallet(
      id: json['id'],
      walletInfo: WalletInfo(
        label: json['wallet']['label'],
        currency: json['wallet']['currency'],
        type: WalletType.parse(json['wallet']['type']),
      ),
    );
  }

  Map<String, dynamic> toMap() {
    return {
      'id': this.id.toString(),
      'wallet': {
        'label': this.walletInfo.label,
        'currency': this.walletInfo.currency,
        'type': this.walletInfo.type.toString()
      }
    };
  }

  @override
  List<Object> get props => [id, walletInfo];

  @override
  String toString() =>
      'Wallet(id: ${id.value.toString()}, walletInfo: ${walletInfo.toString()})';
}
